#include "mainwindow.h"
#include "ui_mainwindow.h"

///Implementation for class MainWindow
///
///---------------------------------------------------------------------------------------------------
/// Basic Methods

/*!
 * \brief Constructor, sets up ui, disable tabs other than login
 * \param parent
 */
MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->tabWidget->setTabEnabled(1, false);
    ui->tabWidget->setTabEnabled(2, false);
}

/*!
 * \brief Destructor
 */
MainWindow::~MainWindow()
{
    delete ui;
}

///---------------------------------------------------------------------------------------------------
/// Methods for loading data to the UI

/*!
 * \brief Adds a VM to the listWidget, sets its id, connects all Quick Action buttons
 * \param id
 * \param name
 */
void MainWindow::addVM(std::string id, std::string name){
    QListWidgetItem* item = new QListWidgetItem(ui->listWidget);
    ComputeWidget* cw = new ComputeWidget();
    cw->setID(id);
    cw->setName(name);
    connect(this, SIGNAL(stateChanged(std::string, std::string)), cw, SLOT(stateChangedHandler(std::string, std::string)));
    emit stateChanged(myController.getState(id), id);
    connect(cw, SIGNAL(renew(std::string)), this, SLOT(renewClicked(std::string)));
    connect(cw, SIGNAL(start(std::string)), this, SLOT(startClicked(std::string)));
    connect(cw, SIGNAL(wakeupsleep(std::string)), this, SLOT(wakeupsleepClicked(std::string)));
    item->setSizeHint(cw->minimumSize());
    ui->listWidget->addItem(item);
    ui->listWidget->setItemWidget(item, cw);
}

/*!
 * \brief loads all data to the UI, which are stored in Controller, DOES NOT COMMUNICATE WITH THE CLOUD SERVER!!
 */
void MainWindow::loadAllData(){
    VM_names = myController.getVMs();
    ui->listWidget->clear();
    for(size_t i = 0; i < VM_names.size(); i++)
        addVM(std::to_string(i), VM_names.at(i));
    ui->lwTemplates->clear();
    for(auto t : myController.getTemplates())
        ui->lwTemplates->addItem(t.c_str());
}

/*!
 * \brief Loads all data for a VM specified by the id, does NOT communicate with the cloud server!!
 * \param id
 */
void MainWindow::loadVMData(std::string id){
    ui->lStatus->setText(myController.getState(id).c_str());
    ComputeData data = myController.getDataByID(id.c_str());
    ui->lCPU->setText(std::to_string(data.cpu).c_str());
    ui->lMemory->setText(std::to_string(data.memory).c_str());
    ui->lCores->setText(std::to_string(data.cores).c_str());
    ui->lHostname->setText((data.hostname + ":" + std::to_string(data.port)).c_str());
    ui->lUsernameVM->setText(data.username.c_str());
    ui->lPasswordVM->setText(data.password.c_str());
}


///---------------------------------------------------------------------------------------------------
/// Button handlers

///Tab 0 - Login Screen

/*!
 * \brief Login Button handler, connects to server, if authentication is succesful enables all tabs, and loads all data
 */
void MainWindow::on_bLogin_clicked()
{
    std::string serverUrl = ui->iCloudServer->text().toStdString();
    myController.connect(serverUrl);
    std::string username = ui->iUsername->text().toStdString();
    std::string password = ui->iPassword->text().toStdString();
    if(!myController.login(username, password)){
        ui->lInfo->setText("Invalid username or password...");
        return;
    }
    ui->tabWidget->setTabEnabled(1, true);
    ui->tabWidget->setTabEnabled(2, true);
    loadAllData();
    ui->tabWidget->setCurrentIndex(1);
}

/// Tab 1 - Virtual Machines

/*!
 * \brief handles Renew button, all static data is refreshed
 * \param id
 */
void MainWindow::renewClicked(std::string id){
    myController.renewByID(id);
    loadVMData(id);
    ui->listWidget->item(std::stoi(id))->setSelected(true);
}

/*!
 * \brief handles Start button (fetches data from server)
 * \param id
 */
void MainWindow::startClicked(std::string id){
    myController.startByID(id);
    loadVMData(id);
    ui->listWidget->item(std::stoi(id))->setSelected(true);
}

void MainWindow::wakeupsleepClicked(std::string id){
    std::string state = myController.getState(id);
    if(state == "active"){ //sleep
        myController.sleepByID(id);
        emit stateChanged(myController.getState(id), id);
        loadVMData(id);
        ui->listWidget->item(std::stoi(id))->setSelected(true);
    }
    else if(state == "suspended" || state == "inactive"){ //wake up
        myController.wakeupByID(id);
        emit stateChanged(myController.getState(id), id);
        loadVMData(id);
        ui->listWidget->item(std::stoi(id))->setSelected(true);
    }
    else{
        throw std::runtime_error("undefined VM STATE!!!");
    }
}

/*!
 * \brief handles Renew button (fetches data from server)
 * \param id
 */
void MainWindow::on_listWidget_clicked(const QModelIndex &index)
{
    loadVMData(std::to_string(index.row()));
}

/*!
 * \brief handles Refresh button, refreshes Controller's data and loads to the UI  (fetches data from server)
 */
void MainWindow::on_bRefresh_clicked()
{
    myController.refresh();
    loadAllData();
}

/*!
 * \brief handles new VM button, Opens a dialog for choosing from templates
 */
void MainWindow::on_bNewVM_clicked()
{
    VMChooserDialog* v = new VMChooserDialog(this);
    emit send_templates(getTemplates(myController.session));
    connect(v, SIGNAL(send_chosen_template(Template)), this, SLOT(receive_chosen_template(Template)));
    v->exec();
}

/*!
 * \brief Slot for dialog, creates new VM by the chosen template
 * \param t
 */
void MainWindow::receive_chosen_template(Template t){
    myController.newCompute(t);
    loadAllData();
}

/*!
 * \brief handler, shows/hides VM password
 */
void MainWindow::on_bShowPassword_clicked()
{
    if(ui->lPasswordVM->echoMode() == QLineEdit::Password){
        ui->lPasswordVM->setEchoMode(QLineEdit::Normal);
        ui->bShowPassword->setText("Hide");
    }
    else if(ui->lPasswordVM->echoMode() == QLineEdit::Normal){
        ui->lPasswordVM->setEchoMode(QLineEdit::Password);
        ui->bShowPassword->setText("Show");
    }
    else
        ui->lPasswordVM->setEchoMode(QLineEdit::Password);
}
